@@ -48,10 +48,10 @@ typedef struct
 {
   GtkStack    *titlebar_container;
   GtkRevealer *titlebar_revealer;
-  GtkEventBox *event_box;
   GtkOverlay  *overlay;
 
-  gulong       motion_notify_handler;
+  GtkEventController *motion_controller;
+  gulong              motion_controller_handler;
 
   guint        fullscreen_source;
   guint        fullscreen_reveal_source;
@@ -128,20 +128,17 @@ revealer_set_reveal_child_now (GtkRevealer *revealer,
   gtk_revealer_set_transition_type (revealer, GTK_REVEALER_TRANSITION_TYPE_SLIDE_DOWN);
 }
 
-static gboolean
-dzl_application_window_event_box_motion (DzlApplicationWindow *self,
-                                         GdkEventMotion       *motion,
-                                         GtkEventBox          *event_box)
+static void
+dzl_application_window_motion_cb (DzlApplicationWindow     *self,
+                                  gdouble                   x,
+                                  gdouble                   y,
+                                  GtkEventControllerMotion *controller)
 {
   DzlApplicationWindowPrivate *priv = dzl_application_window_get_instance_private (self);
-  GtkWidget *widget = NULL;
   GtkWidget *focus;
-  gint x = 0;
-  gint y = 0;
 
   g_assert (DZL_IS_APPLICATION_WINDOW (self));
-  g_assert (motion != NULL);
-  g_assert (GTK_IS_EVENT_BOX (event_box));
+  g_assert (GTK_IS_EVENT_CONTROLLER_MOTION (controller));
 
   /*
    * If we are focused in the revealer, ignore this.  We will have already
@@ -150,28 +147,17 @@ dzl_application_window_event_box_motion (DzlApplicationWindow *self,
   focus = gtk_window_get_focus (GTK_WINDOW (self));
   if (focus != NULL &&
       dzl_gtk_widget_is_ancestor_or_relative (focus, GTK_WIDGET (priv->titlebar_revealer)))
-    return GDK_EVENT_PROPAGATE;
-
-  /* The widget is stored in GdkWindow user_data */
-  gdk_window_get_user_data (motion->window, (gpointer *)&widget);
-  if (widget == NULL)
-    return GDK_EVENT_PROPAGATE;
+    return;
 
   /* If the headerbar is underneath the pointer or we are within a
    * small distance of the edge of the window (and monitor), ensure
    * that the titlebar is displayed and queue our next dismissal.
    */
-  if (dzl_gtk_widget_is_ancestor_or_relative (widget, GTK_WIDGET (priv->titlebar_revealer)) ||
-      gtk_widget_translate_coordinates (widget, GTK_WIDGET (self), motion->x, motion->y, &x, &y))
+  if (y <= SHOW_HEADER_WITHIN_DISTANCE)
     {
-      if (y < SHOW_HEADER_WITHIN_DISTANCE)
-        {
-          gtk_revealer_set_reveal_child (priv->titlebar_revealer, TRUE);
-          dzl_application_window_queue_dismissal (self);
-        }
+      gtk_revealer_set_reveal_child (priv->titlebar_revealer, TRUE);
+      dzl_application_window_queue_dismissal (self);
     }
-
-  return GDK_EVENT_PROPAGATE;
 }
 
 static gboolean
@@ -205,7 +191,7 @@ dzl_application_window_complete_fullscreen (DzlApplicationWindow *self)
   if (priv->fullscreen)
     {
       /* Only listen for motion notify events while in fullscreen mode. */
-      g_signal_handler_unblock (priv->event_box, priv->motion_notify_handler);
+      gtk_event_controller_set_propagation_phase (priv->motion_controller, GTK_PHASE_CAPTURE);
 
       if (titlebar && gtk_widget_is_ancestor (titlebar, GTK_WIDGET (priv->titlebar_container)))
         {
@@ -219,7 +205,7 @@ dzl_application_window_complete_fullscreen (DzlApplicationWindow *self)
   else
     {
       /* Motion events are no longer needed */
-      g_signal_handler_block (priv->event_box, priv->motion_notify_handler);
+      gtk_event_controller_set_propagation_phase (priv->motion_controller, GTK_PHASE_NONE);
 
       if (gtk_widget_is_ancestor (titlebar, GTK_WIDGET (priv->titlebar_revealer)))
         {
@@ -245,11 +231,7 @@ dzl_application_window_real_set_fullscreen (DzlApplicationWindow *self,
 
   priv->fullscreen = fullscreen;
 
-  if (priv->fullscreen_source != 0)
-    {
-      g_source_remove (priv->fullscreen_source);
-      priv->fullscreen_source = 0;
-    }
+  dzl_clear_source (&priv->fullscreen_source);
 
   if (priv->fullscreen)
     {
@@ -308,11 +290,7 @@ dzl_application_window_set_focus (GtkWindow *window,
       if (dzl_gtk_widget_is_ancestor_or_relative (widget, GTK_WIDGET (priv->titlebar_revealer)))
         {
           /* Disable transition while the revealer is focused */
-          if (priv->fullscreen_reveal_source != 0)
-            {
-              g_source_remove (priv->fullscreen_reveal_source);
-              priv->fullscreen_reveal_source = 0;
-            }
+          dzl_clear_source (&priv->fullscreen_reveal_source);
 
           /* If this was just focused, we might need to make it visible */
           gtk_revealer_set_reveal_child (priv->titlebar_revealer, TRUE);
@@ -323,11 +301,7 @@ dzl_application_window_set_focus (GtkWindow *window,
            * the titlebar immediately to get out of the users way.
            */
           gtk_revealer_set_reveal_child (priv->titlebar_revealer, FALSE);
-          if (priv->fullscreen_reveal_source != 0)
-            {
-              g_source_remove (priv->fullscreen_reveal_source);
-              priv->fullscreen_reveal_source = 0;
-            }
+          dzl_clear_source (&priv->fullscreen_reveal_source);
         }
     }
 }
@@ -342,7 +316,7 @@ dzl_application_window_add (GtkContainer *container,
   g_assert (DZL_IS_APPLICATION_WINDOW (self));
   g_assert (GTK_IS_WIDGET (widget));
 
-  gtk_container_add (GTK_CONTAINER (priv->event_box), widget);
+  gtk_container_add (GTK_CONTAINER (priv->overlay), widget);
 }
 
 static gboolean
@@ -367,6 +341,29 @@ dzl_application_window_key_press_event (GtkWidget   *widget,
   return ret;
 }
 
+static gboolean
+dzl_application_window_window_state_event (GtkWidget           *widget,
+                                           GdkEventWindowState *event)
+{
+  DzlApplicationWindow *self = (DzlApplicationWindow *)widget;
+  DzlApplicationWindowPrivate *priv = dzl_application_window_get_instance_private (self);
+  gboolean ret;
+
+  g_assert (DZL_IS_APPLICATION_WINDOW (self));
+  g_assert (event != NULL);
+
+  ret = GTK_WIDGET_CLASS (dzl_application_window_parent_class)->window_state_event (widget, event);
+
+  /* Clear our fullscreen state if the window-manager un-fullscreened us.
+   * This fixes an issue on gnome 3.30 (and other window managers) that can
+   * clear the fullscreen state out from under the app.
+   */
+  if (priv->fullscreen && (event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN) == 0)
+    dzl_application_window_set_fullscreen (self, FALSE);
+
+  return ret;
+}
+
 static void
 dzl_application_window_destroy (GtkWidget *widget)
 {
@@ -375,15 +372,10 @@ dzl_application_window_destroy (GtkWidget *widget)
 
   g_assert (DZL_IS_APPLICATION_WINDOW (self));
 
-  if (priv->event_box != NULL)
-    {
-      g_signal_handler_disconnect (priv->event_box, priv->motion_notify_handler);
-      priv->motion_notify_handler = 0;
-    }
+  g_clear_object (&priv->motion_controller);
 
   g_clear_pointer ((GtkWidget **)&priv->titlebar_container, gtk_widget_destroy);
   g_clear_pointer ((GtkWidget **)&priv->titlebar_revealer, gtk_widget_destroy);
-  g_clear_pointer ((GtkWidget **)&priv->event_box, gtk_widget_destroy);
   g_clear_pointer ((GtkWidget **)&priv->overlay, gtk_widget_destroy);
 
   dzl_clear_source (&priv->fullscreen_source);
@@ -443,6 +435,7 @@ dzl_application_window_class_init (DzlApplicationWindowClass *klass)
 
   widget_class->destroy = dzl_application_window_destroy;
   widget_class->key_press_event = dzl_application_window_key_press_event;
+  widget_class->window_state_event = dzl_application_window_window_state_event;
 
   container_class->add = dzl_application_window_add;
 
@@ -490,6 +483,7 @@ dzl_application_window_init (DzlApplicationWindow *self)
   priv->overlay = g_object_new (GTK_TYPE_OVERLAY,
                                 "visible", TRUE,
                                 NULL);
+  gtk_widget_set_events (GTK_WIDGET (priv->overlay), GDK_POINTER_MOTION_MASK);
   g_signal_connect (priv->overlay,
                     "destroy",
                     G_CALLBACK (gtk_widget_destroyed),
@@ -497,23 +491,13 @@ dzl_application_window_init (DzlApplicationWindow *self)
   GTK_CONTAINER_CLASS (dzl_application_window_parent_class)->add (GTK_CONTAINER (self),
                                                                   GTK_WIDGET (priv->overlay));
 
-  priv->event_box = g_object_new (GTK_TYPE_EVENT_BOX,
-                                  "above-child", FALSE,
-                                  "visible", TRUE,
-                                  "visible-window", TRUE,
-                                  NULL);
-  gtk_widget_set_events (GTK_WIDGET (priv->event_box), GDK_POINTER_MOTION_MASK);
-  g_signal_connect (priv->event_box,
-                    "destroy",
-                    G_CALLBACK (gtk_widget_destroyed),
-                    &priv->event_box);
-  priv->motion_notify_handler =
-    g_signal_connect_swapped (priv->event_box,
-                              "motion-notify-event",
-                              G_CALLBACK (dzl_application_window_event_box_motion),
+  priv->motion_controller = gtk_event_controller_motion_new (GTK_WIDGET (priv->overlay));
+  priv->motion_controller_handler =
+    g_signal_connect_swapped (priv->motion_controller,
+                              "motion",
+                              G_CALLBACK (dzl_application_window_motion_cb),
                               self);
-  g_signal_handler_block (priv->event_box, priv->motion_notify_handler);
-  gtk_container_add (GTK_CONTAINER (priv->overlay), GTK_WIDGET (priv->event_box));
+  gtk_event_controller_set_propagation_phase (priv->motion_controller, GTK_PHASE_NONE);
 
   priv->titlebar_revealer = g_object_new (GTK_TYPE_REVEALER,
                                           "valign", GTK_ALIGN_START,
