@@ -82,6 +82,12 @@ typedef struct
    */
   GNode *root;
 
+  /*
+   * GHashTable to match command/action to a nodedata, useful to generate
+   * a tooltip-text string for a given widget.
+   */
+  GHashTable *command_id_to_node_data;
+
   /*
    * We keep track of the search paths for loading themes here. Each element is
    * a string containing the path to the file-system resource. If the path
@@ -302,6 +308,8 @@ dzl_shortcut_manager_finalize (GObject *object)
   DzlShortcutManager *self = (DzlShortcutManager *)object;
   DzlShortcutManagerPrivate *priv = dzl_shortcut_manager_get_instance_private (self);
 
+  g_clear_pointer (&priv->command_id_to_node_data, g_hash_table_unref);
+
   if (priv->root != NULL)
     {
       g_node_traverse (priv->root, G_IN_ORDER, G_TRAVERSE_ALL, -1, free_node_data, NULL);
@@ -449,6 +457,7 @@ dzl_shortcut_manager_init (DzlShortcutManager *self)
 {
   DzlShortcutManagerPrivate *priv = dzl_shortcut_manager_get_instance_private (self);
 
+  priv->command_id_to_node_data = g_hash_table_new (g_str_hash, g_str_equal);
   priv->seen_entries = g_hash_table_new (shortcut_entry_hash, NULL);
   priv->themes = g_ptr_array_new_with_free_func (destroy_theme);
   priv->root = g_node_new (NULL);
@@ -812,7 +821,6 @@ dzl_shortcut_manager_run_fallbacks (DzlShortcutManager     *self,
                                     const DzlShortcutChord *chord)
 {
   DzlShortcutManagerPrivate *priv = dzl_shortcut_manager_get_instance_private (self);
-  static DzlShortcutChord *inspector_chord;
 
   DZL_ENTRY;
 
@@ -866,23 +874,6 @@ dzl_shortcut_manager_run_fallbacks (DzlShortcutManager     *self,
             DZL_RETURN (TRUE);
         }
 
-      /*
-       * If we this is the ctrl+shift+d keybinding to activate the inspector,
-       * then try to see if we should handle that manually.
-       */
-      if G_UNLIKELY (inspector_chord == NULL)
-        inspector_chord = dzl_shortcut_chord_new_from_string ("<ctrl><shift>d");
-      if (dzl_shortcut_chord_equal (chord, inspector_chord))
-        {
-          g_autoptr(GSettings) settings = g_settings_new ("org.gtk.Settings.Debug");
-
-          if (g_settings_get_boolean (settings, "enable-inspector-keybinding"))
-            {
-              gtk_window_set_interactive_debugging (TRUE);
-              DZL_RETURN (TRUE);
-            }
-        }
-
       /*
        * Now fallback to trying to activate the action within GtkApplication
        * as the legacy Gtk bindings would do.
@@ -1310,6 +1301,7 @@ dzl_shortcut_manager_add_action (DzlShortcutManager *self,
                                  const gchar        *title,
                                  const gchar        *subtitle)
 {
+  DzlShortcutManagerPrivate *priv;
   DzlShortcutNodeData *data;
   GNode *parent;
 
@@ -1320,6 +1312,8 @@ dzl_shortcut_manager_add_action (DzlShortcutManager *self,
   if (self == NULL)
     self = dzl_shortcut_manager_get_default ();
 
+  priv = dzl_shortcut_manager_get_instance_private (self);
+
   section = g_intern_string (section);
   group = g_intern_string (group);
   title = g_intern_string (title);
@@ -1338,6 +1332,8 @@ dzl_shortcut_manager_add_action (DzlShortcutManager *self,
 
   g_node_append_data (parent, data);
 
+  g_hash_table_insert (priv->command_id_to_node_data, (gpointer)data->name, data);
+
   g_signal_emit (self, signals [CHANGED], 0);
 }
 
@@ -1349,6 +1345,7 @@ dzl_shortcut_manager_add_command (DzlShortcutManager *self,
                                   const gchar        *title,
                                   const gchar        *subtitle)
 {
+  DzlShortcutManagerPrivate *priv;
   DzlShortcutNodeData *data;
   GNode *parent;
 
@@ -1359,6 +1356,8 @@ dzl_shortcut_manager_add_command (DzlShortcutManager *self,
   if (self == NULL)
     self = dzl_shortcut_manager_get_default ();
 
+  priv = dzl_shortcut_manager_get_instance_private (self);
+
   section = g_intern_string (section);
   group = g_intern_string (group);
   title = g_intern_string (title);
@@ -1377,6 +1376,8 @@ dzl_shortcut_manager_add_command (DzlShortcutManager *self,
 
   g_node_append_data (parent, data);
 
+  g_hash_table_insert (priv->command_id_to_node_data, (gpointer)data->name, data);
+
   g_signal_emit (self, signals [CHANGED], 0);
 }
 
@@ -1667,3 +1668,46 @@ dzl_shortcut_manager_merge (DzlShortcutManager *self,
 
   DZL_EXIT;
 }
+
+/**
+ * _dzl_shortcut_manager_get_command_info:
+ * @self: a #DzlShortcutManager
+ * @command_id: the command-id
+ * @title: (out) (optional): a location for the title
+ * @subtitle: (out) (optional): a location for the subtitle
+ *
+ * Gets command information about command-id
+ *
+ * Returns: %TRUE if the command-id was found and out parameters were set.
+ *
+ * Since: 3.32
+ */
+gboolean
+_dzl_shortcut_manager_get_command_info (DzlShortcutManager  *self,
+                                        const gchar         *command_id,
+                                        const gchar        **title,
+                                        const gchar        **subtitle)
+{
+  DzlShortcutManagerPrivate *priv;
+  DzlShortcutNodeData *node;
+
+  if (self == NULL)
+    self = dzl_shortcut_manager_get_default ();
+
+  g_return_val_if_fail (DZL_IS_SHORTCUT_MANAGER (self), FALSE);
+
+  priv = dzl_shortcut_manager_get_instance_private (self);
+
+  if ((node = g_hash_table_lookup (priv->command_id_to_node_data, command_id)))
+    {
+      if (title != NULL)
+        *title = node->title;
+
+      if (subtitle != NULL)
+        *subtitle = node->subtitle;
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
